'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const defu = require('defu');
const useCookies = require('@vueuse/integrations/useCookies');
const core = require('@vueuse/core');
const ohmyfetch = require('ohmyfetch');
const vue = require('vue');
const vueRouter = require('vue-router');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const defu__default = /*#__PURE__*/_interopDefaultLegacy(defu);

const useToken = (options) => {
  const { token } = options.local;
  const key = `auth.${token.prefix}.local`;
  const cookies = useCookies.useCookies();
  const getToken = () => cookies.get(key);
  const setToken = (tokenValue) => {
    const parseToken = `${token.type} ${tokenValue}`;
    cookies.set(key, parseToken);
  };
  const removeToken = () => cookies.remove(key);
  return { getToken, setToken, removeToken };
};
const useToken$1 = useToken;

const useUser = core.createSharedComposable((options) => {
  const { getToken, removeToken } = useToken$1(options);
  const { baseUrl, local } = options;
  const { token, endpoints } = local;
  const { user } = endpoints;
  const { propertyInFetch, propertyRole, propertyPermission } = local.user;
  const state = vue.reactive({
    user: null,
    loggedIn: false
  });
  const isLoggedIn = () => state.loggedIn;
  const getUser = () => state.user;
  const setUser = (user2) => {
    state.user = user2;
    state.loggedIn = true;
  };
  const getRole = () => {
    if (!state.user)
      return "";
    return state.user[propertyRole];
  };
  const isRole = (role) => {
    const roleUser = getRole();
    return roleUser === role;
  };
  const getPermissions = () => {
    if (!state.user)
      return [];
    return state.user[propertyPermission] || [];
  };
  const hasPermission = (permission) => {
    const permissionsUser = getPermissions();
    return permissionsUser.includes(permission);
  };
  const fetchUser = async () => {
    if (!user)
      return;
    const url = baseUrl + user.url;
    try {
      const { data } = await ohmyfetch.$fetch(url, {
        method: user.method,
        async onRequest({ options: options2 }) {
          options2.headers = {
            [token.name]: getToken()
          };
        }
      });
      const _user = propertyInFetch ? data[propertyInFetch] : data;
      setUser(_user);
      return _user;
    } catch (e) {
      resetState();
      removeToken();
      throw e;
    }
  };
  const resetState = () => {
    state.user = null;
    state.loggedIn = false;
  };
  return {
    state,
    getUser,
    setUser,
    getRole,
    isRole,
    getPermissions,
    hasPermission,
    isLoggedIn,
    fetchUser,
    resetState
  };
});
const useUser$1 = useUser;

const DEFAULT_OPTION = {
  fullPathRedirect: true,
  watchLoggedIn: true,
  redirect: {
    login: "/login",
    logout: "/login",
    home: "/"
  },
  local: {
    endpoints: {
      login: { url: "/api/auth/login", method: "post" },
      logout: { url: "/api/auth/logout", method: "post" },
      user: { url: "/api/auth/user", method: "get" }
    },
    token: {
      property: "token",
      type: "Bearer",
      name: "Authorization",
      prefix: "_token"
    },
    user: {
      propertyInLogin: "user",
      propertyInFetch: "",
      propertyRole: "role",
      propertyPermission: "permissions",
      autoFetch: true
    }
  }
};

const isObject = (val) => toString.call(val) === "[object Object]";

const createAuth = (app, _options) => {
  const options = defu__default(_options, DEFAULT_OPTION);
  app.config.globalProperties.authOptions = options;
  const { router, redirect, fullPathRedirect, local } = options;
  const { getToken } = useToken$1(options);
  const { isLoggedIn, fetchUser, isRole, hasPermission } = useUser$1(options);
  router.beforeEach(async (to, from, next) => {
    const token = getToken();
    const { auth: metaAuth } = to.meta;
    const isValidateAuthenticatedNextPage = (metaAuth2) => {
      let isAuthenticated = false;
      if (metaAuth2 === true && isLoggedIn()) {
        isAuthenticated = true;
      }
      if (isObject(metaAuth2)) {
        if (metaAuth2.hasOwnProperty("role")) {
          isAuthenticated = isRole(metaAuth2.role);
        }
        if (metaAuth2.hasOwnProperty("permission")) {
          isAuthenticated = hasPermission(metaAuth2.permission);
        }
      }
      return isAuthenticated;
    };
    const redirectToLoginPage = () => {
      const redirectPath = fullPathRedirect ? to.fullPath : to.path;
      next(`${redirect.login}?redirect=${redirectPath}`);
    };
    if (token) {
      if (to.path === redirect.login) {
        return next({ path: redirect.home });
      }
      if (!isLoggedIn() && local.user.autoFetch) {
        try {
          await fetchUser();
        } catch {
          redirectToLoginPage();
        }
      }
      if (!metaAuth || isValidateAuthenticatedNextPage(metaAuth)) {
        return next();
      }
      return next(`${redirect.home}`);
    }
    if (redirect.login === to.path || !metaAuth || isValidateAuthenticatedNextPage(metaAuth)) {
      return next();
    }
    return redirectToLoginPage();
  });
};
const authPlugin = {
  install: createAuth
};
const authPlugin$1 = authPlugin;

const useAuth = (_options) => {
  const router = vueRouter.useRouter();
  const route = vueRouter.useRoute();
  const instance = vue.getCurrentInstance();
  const options = _options || instance?.appContext.config.globalProperties.authOptions;
  const { baseUrl, redirect, watchLoggedIn } = options;
  const { endpoints, token, user } = options.local;
  const {
    state,
    getUser,
    setUser,
    fetchUser,
    getRole,
    isRole,
    getPermissions,
    hasPermission,
    resetState
  } = useUser$1(options);
  const { setToken, getToken, removeToken } = useToken$1(options);
  const redirectAfterLogin = () => {
    const path = route.query.redirect || redirect.home;
    router.replace(path);
  };
  const redirectAfterLogout = () => {
    const isNeedRedirect = route.meta.auth;
    if (isNeedRedirect) {
      const path = redirect.logout || redirect.home;
      router.replace(path);
    }
  };
  const login = async (args) => {
    if (!endpoints.login) {
      return;
    }
    const url = baseUrl + endpoints.login.url;
    const { data } = await ohmyfetch.$fetch(url, {
      method: endpoints.login.method,
      body: args
    });
    const _token = data[token.property];
    const _user = data[user.propertyInLogin];
    _token && setToken(_token);
    if (_user) {
      setUser(_user);
    } else {
      await fetchUser();
    }
    redirectAfterLogin();
  };
  const logout = async (args) => {
    removeToken();
    resetState();
    if (watchLoggedIn) {
      redirectAfterLogout();
    }
    if (!endpoints.logout) {
      return;
    }
    const url = baseUrl + endpoints.logout.url;
    const { data } = await ohmyfetch.$fetch(url, {
      method: endpoints.logout.method,
      body: args
    });
    return data;
  };
  return {
    login,
    logout,
    setToken,
    getToken,
    getUser,
    setUser,
    getRole,
    isRole,
    getPermissions,
    hasPermission,
    resetState,
    fetchUser,
    ...vue.toRefs(state)
  };
};
const useAuth$1 = useAuth;

exports.DEFAULT_OPTION = DEFAULT_OPTION;
exports.authPlugin = authPlugin$1;
exports.useAuth = useAuth$1;
